#include "pw/scene/node.hpp"

namespace pw {

node::node(const std::string &name)
    : _name(name)
{
}

node::node(const node &node)
	: _children(node.children())
	, _parents(node._parents)
	, _name(node._name)
{
}

node *node::clone(const unsigned short &copymode) const
{
    return nullptr;
}

std::string node::name() const
{
    return _name;
}

void node::set_name(const std::string &name)
{
    _name = name;
}

std::shared_ptr<node> node::add_child(std::shared_ptr<node> anode)
{
	anode->add_parent(this);
	_children.push_back(anode);
	return anode;
}

void node::add_parent(node *anode)
{
	_parents.push_back(anode);
}

void node::visit(visitor& visitor)
{
	visitor.enter(this);

	if (visitor.direction == node::visitor::direction::up)
		this->ascend(visitor);
	else {
		this->descend(visitor);
	}

	visitor.leave(this);
}

void node::ascend(visitor& visitor)
{
	for (auto p : this->parents()) p->visit(visitor);
}

void node::descend(visitor& visitor)
{
	for (auto c : this->children()) c->visit(visitor);
}

std::shared_ptr<node> node::shared()
{
	return std::shared_ptr<node>(this);
}

node::~node()
{
    _children.clear();
    _parents.clear();
}




//
//
//

void node::register_component(component *c) {
    _components.push_back(std::shared_ptr<component>(c));
}

void node::unregister_component(component *c)
{
//    component::array::iterator it = _components.end();

//    while ((it = std::find(_components.begin(),_components.end(),std::shared_ptr<component>(c))) != _components.end()) {
//        _components.erase(it);
//    }
}

const component::array& node::components() const
{
    return _components;
}




}
